<?php


namespace Service;


use Doctrine\ORM\EntityManager;
use Exception;
use Teknisa\Model\Teknisa\Entities\CpCall;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Zeedhi\Framework\ORM\DateTime;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;

class Issue
{
    const ACTIVE = 'A';
    const IS_ISSUE = 'Y';
    const IS_TASK = 'Y';
    const FIRST_COMMENTS = 'Abertura de issue.';
    /**
     * @var EntityManager
     */
    public $entityManager;
    /**
     * @var Utilities
     */
    public $environment;

    public function __construct()
    {
        $this->entityManager  = InstanceProvider::getEntityManager();
        $this->environment    = InstanceProvider::getEnvironment();
    }

    public function createIssueByProductCall($reference, $product, $classification, $description, $board, $reporterName,
                                             $reporterFrom,$sponsorId,$reportedBy)
    {
        $description = $reporterName . " - " . $reporterFrom . PHP_EOL . PHP_EOL . PHP_EOL . $description;

        $now = new \DateTime();
        $dueDate = $now->format('d/m/Y');
        $issueStatus = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'TaskStatus')
            ->findOneBy(array('control' => Entities\TaskStatus::TASK_STATUS_CONTROL_BACKLOG))->getId();
        $reporterId = InstanceProvider::getEnvironment()->getCurrentUserId();
        $priority = null;
        $percentCompleted = 0;
        $followers = array();
        $labels = null;

        try {
            return $this->saveNewIssue($reference, $classification, $issueStatus, $reporterId, $dueDate, $priority, $percentCompleted, $description,
                $board, $followers, $labels, $sponsorId);
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function saveNewIssue($name, $classification, $issueStatus, $reporter, $dueDate, $priority, $percentCompleted,
                                 $description, $board, $followers, $labels = null, $sponsor = null, $endDate = null,
                                 $productVersionId = null, $version = null, $projectId = null, $versionDev = null,
                                 $productVersionDevId = null, $isBoardIssue = null) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $name = substr($name, 0, 200);
            $issueStatus 	= $this->getIssueStatus($issueStatus);
            $board 			= $this->entityManager->find(Utilities::ENTITY_DIR.'Project', $board);
            if($endDate){
                $endDate = \DateTime::createFromFormat('d/m/Y', $endDate);
            }
            if($dueDate) {
                $dueDate = \DateTime::createFromFormat('d/m/Y', $dueDate);
            }else {
                $dueDate = new \DateTime();
            }
            if($priority){
                $priority = $this->entityManager->find(Utilities::ENTITY_DIR . 'Priority', $priority);
            }
            /** @var Entities\Classification $classification */
            $classification = InstanceProvider::getRepository()->getClassification()->find($classification);
            $reporter 		= InstanceProvider::getRepository()->getResource()->find($reporter);
            if($isBoardIssue){
                $productVersion = $this->getProductVersion($productVersionId);
                $productVersionDev = $this->getProductVersion($productVersionDevId);
            } else {
                $productVersion = $this->getProductVersion($productVersionId);
                $productVersionDev = $this->getProductVersionDev($projectId, $version, $productVersionId);
            }

            $issue  = InstanceProvider::getFactory()->getTask()->createTask($board, null, $name, $this->environment->getCurrentTime(),
                $dueDate, self::ACTIVE, $description, $percentCompleted, null, null, $issueStatus,
                null, null, self::IS_ISSUE, 'N', $classification, null, $priority, null, null, $endDate, null, null, $productVersion, $productVersionDev);

            $this->entityManager->persist($issue);

            $allocationService = InstanceProvider::getService()->getAllocation();
            if(!empty($reporter)){
                $allocationService->createAllocation($issue, $reporter, Entities\Allocation::TYPE_REPORTER);
            }

            if(!empty($sponsor)){
                $sponsorEntity = $this->entityManager->find(Utilities::ENTITY_DIR.'Resource', $sponsor);
                $allocationService->createAllocation($issue, $sponsorEntity, Entities\Allocation::TYPE_SPONSOR);
            }

            $this->setFollowers($followers, $issue);

            if(is_array($labels)){
                $this->setLabels($labels,$issue);
            }
            $this->entityManager->flush();

            $commentType = InstanceProvider::getRepository()->getCommentType()->findOneBy(array('control' => Entities\CommentType::TYPE_OPEN_ISSUE));
            $fistComments = InstanceProvider::getFactory()->getComments()->createComments($issue,self::FIRST_COMMENTS, $commentType);
            $issue->setCurrentTaskStatusIn(Utilities::getEnvironment()->getCurrentTime());
            $this->entityManager->persist($issue);
            $this->entityManager->persist($fistComments);
            $this->viewCommentsSaveData($issue->getId());
            $this->entityManager->flush();

            $reporterAllocation = $allocationService->getAllocationByUserAndTask($reporter->getId(), $issue->getId());
            if(!empty($reporterAllocation)){
                $reporterAllocation->setLastPostView($this->environment->getCurrentTime());
                $this->entityManager->persist($reporterAllocation);
                $this->entityManager->flush();
            }

            $connection->commit();
            self::createCall($name,$reporter,$description,
                $issue->getId(),$issue->getProject()->getId(),$classification->getId());

            $this->sendOpenIssueMail($issue);
            return $issue->getId();
        }
        catch(Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    private function getIssueStatus($issueStatus)
    {
        $taskStatusRepository = InstanceProvider::getRepository()->getTaskStatus();
        if($issueStatus){
            return $taskStatusRepository->find($issueStatus);
        } else {
            return $taskStatusRepository->findOneBy(array('CONTROL' => Entities\TaskStatus::TASK_STATUS_CONTROL_BACKLOG));
        }
    }

    public function getProductVersion($productVersionId)
    {
        if($productVersionId){
            return InstanceProvider::getRepository()->getProductVersion()->find($productVersionId);
        }
        return null;
    }

    public function getProductVersionDev($projectId, $version, $productVersionId)
    {
        if($productVersionId){
            $productVersionDevId = $this->getProductVersionDevFromProductVersion($version, $projectId);
            if($productVersionDevId){
                return InstanceProvider::getRepository()->getProductVersion()->find($productVersionDevId);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    public function getProductVersionDevFromProductVersion($version, $projectId)
    {
        $product = InstanceProvider::getRepository()->getProduct()->findOneBy(array('project' => $projectId));
        $organizationId = InstanceProvider::getEnvironment()->getOrganizationId();
        $versions = explode('.', $version);
        $version = $versions[0];
        $revision = $versions[1];
        $productVersionsLts = InstanceProvider::getRepository()->getProductVersion()->findLtsCurrentProductVersionFromVersion($product->getId(), $organizationId, $version, $revision);
        return !empty($productVersionsLts) ? $productVersionsLts[0]['ID'] : null;
    }

    protected function setFollowers(array $followers, $issue)
    {
        $allocationService = InstanceProvider::getService()->getAllocation();
        $resourceRepository = InstanceProvider::getRepository()->getResource();
        $allocationRepository = InstanceProvider::getRepository()->getAllocation();
        foreach ($followers as $eachResource) {
            if (!empty($eachResource)) {
                $resourceProject = $resourceRepository->find($eachResource);
                $allocation = $allocationRepository->findBy(array("task" => $issue->getId(), "resource" => $resourceProject->getId(),
                    'type' => Allocation::FOLLOWER));
                if (empty($allocation)) {
                    $allocationService->createAllocation($issue, $resourceProject, Allocation::FOLLOWER);
                }
            }
        }
        $this->entityManager->flush();
    }

    public static function createCall($name,Resource $reportedBy,$description,$issueId,$projectId,$classificationId)
    {
        $call = InstanceProvider::getRepository()->getCpCall()->findOneBy(array('callRelationship'=> $issueId));
        if(!$call){
            $cdOperador = $reportedBy->getExternalCode();
            $call = self::factoryCall($name,$cdOperador,$description,$issueId,$projectId,$classificationId);
            $entityManager = InstanceProvider::getEntityManager();
            $entityManager->persist($call);
            $entityManager->flush();
        }
    }

    public static function factoryCall($reference, $reportedBy, $description, $callRelationship, $product,
                                       $classification)
    {
        $entity = new  CpCall();
        $entity->setReference($reference);
        if (!$reportedBy) {
            $reportedBy = 1;
        }
        $entity->setReportedBy($reportedBy);
        if (!$description) {
            $description = ' ';
        }
        $entity->setDescription($description);
        $entity->setType(CpCall::TYPE_ISSUE);
        $entity->setCallRelationship($callRelationship);
        $entity->setProduct($product);
        $entity->setClassification($classification);
        return $entity;
    }


    public function sendOpenIssueMail(Entities\Task $issue)
    {
        if($issue->getProject()->getNotifyIssueOpening() == Entities\Project::IS_NOTIFY_ISSUE){
            $commenterId = $this->environment->getCurrentUserId();
            $emails    = InstanceProvider::getRepository()->getTask()->getManagersEmailsByIssueId($issue->getId(), $commenterId);
            $issueName = $issue->getName();
            $reporter  = DevOpsUtilities::getResourceNameForSendingMail($issue->getId(), Entities\Allocation::TYPE_REPORTER);
            $sponsor   = DevOpsUtilities::getResourceNameForSendingMail($issue->getId(), Entities\Allocation::TYPE_SPONSOR);
            $organizationName = InstanceProvider::getRepository()->getOrganization()->find($issue->getOrganization())->getName();
            $commenterName = $this->environment->getCurrentUser()->getName();
            $date = $this->environment->getCurrentTime()->format(Utilities::FORMAT_DATE_DEFAULT_NO_SECS);
            $issueTitle  = $issue->getId() . " - " . $issueName;
            $productName = $issue->getProject()->getName();

            foreach ($emails as $email) {
                InstanceProvider::getMailIssueMail()->sendOpenIssueMail($email['EMAIL'], $issueTitle, $commenterName, $date, $issue->getDescription(),
                    $reporter, $sponsor, $issueName, $issue->getId(), $productName, $organizationName);
            }
        }
    }

    public function viewCommentsSaveData($taskId){
        $user = $this->environment->getCurrentUser();
        /** @var Entities\Task $issue */
        $issue = InstanceProvider::getRepository()->getTask()->find($taskId);
        $lastDate = null;
        if($issue){
            /** @var Entities\Allocation $allocation */
            $allocations = InstanceProvider::getRepository()->getAllocation()->findBy(array('task' => $issue ,'resource' => $user));
            if(empty($allocations)){
                $allocations = $this->addFollower($issue, $user);
                $allocations->setLastPostView($this->environment->getCurrentTime());
                $this->entityManager->persist($allocations);
                $this->entityManager->flush();
            }else{
                foreach($allocations as $allocation){
                    $allocation->setLastPostView($this->environment->getCurrentTime());
                    $this->entityManager->persist($allocation);
                    $this->entityManager->flush();
                    $lastDate = $allocation->getLastPostView();
                }
            }
        }
        return $lastDate;
    }

    public function addFollower(Entities\Task $issue, Resource $resource){
        return InstanceProvider::getService()->getAllocation()->createAllocation($issue, $resource, Allocation::FOLLOWER);
    }

    public function findIssue($issueId,$accompanyingIssue = false){
        $issues = InstanceProvider::getRepository()->getTask()->getIssue($issueId, $this->environment->getCurrentUserId());
        foreach($issues as &$issue) {
            $issue['FOLLOWERS'] = Utilities::setSelectMultiInQuery($issue['FOLLOWERS'],true);

            $issue['LABELS'] =  Utilities::setSelectMultiInQuery($issue['LABELS']);

            $issue['CLIENT_ACCEPT'] = Entities\TaskStatus::TASK_STATUS_CONTROL_CLIENT_ACCEPT == $issue['CONTROL'];

            $issue['TASK_CLASSIFICATION_IDS'] =  Utilities::setSelectMultiInQuery($issue['TASK_CLASSIFICATION_IDS']);

            $issue['ACTIONS_OF_EXITS'] = $this->getActionsOfExits($issueId);

            list($issue['LABELS'], $issue['LABELS_NOT_EDITABLE']) =
                $this->getLabelsForEditAndNotEditInManagerIssue($issue['LABELS'], $issue['BOARD'],$accompanyingIssue);

            $issue['ACTIONS_REDO_BUDGET'] = $this->allowRedoBudget($issueId);
        }
        return $issues;
    }


    public function allowRedoBudget($issueId){
        $return = false;
        $budgetRepository = InstanceProvider::getRepository()->getBudget();
        /** @var Entities\Budget $budget */
        $budget = $budgetRepository->findOneBy(array('task' => $issueId));
        if($budget && !$budget->getExternalId()){
            $organizationId = \Util\Utilities::getEnvironment()->getCurrentOrganizationId();
            $urlSm = $budgetRepository->getUrlIntegrationSM($organizationId);
            if($urlSm &&
                $budget->getTask()->getTaskStatus() &&
                $budget->getTask()->getTaskStatus()->getControl() == Entities\TaskStatus::TASK_STATUS_CONTROL_BACKLOG){

                /** @var Entities\Comments $comment */
                $comment = InstanceProvider::getRepository()->getComments()->findOneBy(array('task' => $issueId ,'commentType' => 3),array('createdAt'=> 'DESC'));
                if($comment){
                    $classificationId = $budget->getTask()->getClassification()->getId();
                    $taskSprint = InstanceProvider::getRepository()->getTask()->getTaskRelationshipByIssue($issueId,  " AND CLASSIFICATION_ID = $classificationId AND PERCENT <> 100");
                    if(!empty($taskSprint)){
                        $return = true;
                    }
                }
            }
        }
        return $return;
    }

    /**
     * @return array
     */
    public function getImInMeetingProjectIds()
    {
        $projectType = Utilities::getProjectTypeMeeting();
        /** @var Entities\ResourceProject[] $projectResources */
        $meetingProject = InstanceProvider::getRepository()->getProject()->findBy(array('projectType' => $projectType));;
        $projectResources = InstanceProvider::getRepository()->getResourceProject()->findBy(array(
            'resourceC' => $this->environment->getCurrentUserId(),
            'project' => $meetingProject));
        $projectIds = Utilities::getArrayByArrayObjectWithGet($projectResources,array('getProject','getId'));

        return $projectIds;
    }

    public function getLabelsForEditAndNotEditInManagerIssue(array $labelIds,$board = null, $accompanyingIssue = false, $editable = array(),
                                                             $notEditable = array())
    {
        $meetingProjectIds = $this->getImInMeetingProjectIds();
        if($board && !$accompanyingIssue){
            $meetingProjectIds[] = $board;
        }
        $userId = $this->environment->getCurrentUserId();
        foreach ($labelIds as $labelId) {
            /** @var Entities\Label $label */
            $label = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Label')->find($labelId);
            $relatedResource = $label->getResource();
            if ($relatedResource) {
                if ($relatedResource->getId() == $userId) {
                    $editable[] = $labelId;
                } else {
                    $notEditable[] = $labelId;
                }
            } else if ($label->getProject() && in_array($label->getProject()->getId(), $meetingProjectIds)) {
                $editable[] = $labelId;
            } else {
                $notEditable[] = $labelId;
            }
        }
        return array($editable,$notEditable);
    }

    public function getActionsOfExits($issueId): array
    {
        $actionsOfExits = InstanceProvider::getRepository()->getTask()->getActionsOfExits($issueId);

        /** @var Entities\Task $issue */
        $issue = InstanceProvider::getRepository()->getTask()->find($issueId);

        foreach ($actionsOfExits as &$actionsOfExit) {
            $actionsOfExit['COMMENT_DESCRIPTION'] = $actionsOfExit['COMMENT_DESCRIPTION']?$actionsOfExit['COMMENT_DESCRIPTION']:'';
        }

        if ($issue->getTaskStatus()->getControl() === 'H') {
            $actionsOfExits = array_merge($actionsOfExits, $this->getActionsFinishOrDecline());
        }

        return $actionsOfExits;
    }

    /**
     * @return array[]
     */
    private function getActionsFinishOrDecline()
    {
        return array(
            array(
                'ID' => null,
                'ACTION_LABEL' => 'Finalizar Issue',
                'TASK_TYPE_TARGET' => null,
                'COMMENT_TYPE_ID' => null,
                'ATTACHMENT_REQUIRED' => 'N',
                'COMMENT_DESCRIPTION' => '',
                'HOMOLOGATED' => 'Y'
            ),
            array(
                'ID' => null,
                'ACTION_LABEL' => 'Recusar Issue',
                'TASK_TYPE_TARGET' => null,
                'COMMENT_TYPE_ID' => null,
                'ATTACHMENT_REQUIRED' => 'N',
                'COMMENT_DESCRIPTION' => null,
                'HOMOLOGATED' => 'Y'
            ),
        );
    }

    /**
     * @param $issueId
     * @param $reference
     * @param $product
     * @param $classification
     * @param $description
     * @param $board
     * @param Resource $resource
     * @param $resourceSponsorId
     * @return mixed
     * @throws Exception
     */
    public function updateIssueByProductCall($issueId, $reference, $product, $classification, $description, $board, Resource $resource, $resourceSponsorId) {

        $now = new \DateTime();
        $dueDate = $now->format('d/m/Y');
        $issue              = $this->findIssue($issueId)[0];
        $issueStatus        = $issue['ISSUE_STATUS'];
        $reporter           = $resource->getId();
        $priority           = $issue['PRIORITY'];
        $percentCompleted   = $issue['PERCENT_COMPLETED'];
        $followers          = $issue['FOLLOWERS'];
        $sponsor            = $resourceSponsorId;
        $labels             = $issue['LABELS'];
        try{
            $issueId = $this->saveIssue($issueId, $reference, $classification, $issueStatus, $reporter, $dueDate, $priority, $percentCompleted,
                $description, $followers, null, $board, $labels, $sponsor);

              return $this->findIssue($issueId['ID'])[0];
        } catch(Exception $e){
            throw $e;
        }
    }

    public function saveIssue(
        $id, $name, $classification, $issueStatus, $reporter, $dueDate, $priority,
        $percentCompleted, $description, $followers, $endDate, $board = null,
        $labels = null, $sponsor = null, $labelsNotEditable = array(), $isRelevant = Entities\Allocation::NOT_RELEVANT,
        $productVersionId = null, $version = null, $projectId = null, $isBoardIssue = null, $productVersionDevId = null, $issueResponse = null
    ){

        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();

            $name = substr($name, 0, 200);
            $resourceRepository = InstanceProvider::getRepository()->getResource();
            $issue          = InstanceProvider::getRepository()->getTask()->find($id);
            $issueStatus    = $this->entityManager->find(Utilities::ENTITY_DIR . 'TaskStatus', $issueStatus);
            $reporter       = empty($reporter) ? null : $resourceRepository->find($reporter);
            $dueDate        = empty($dueDate) ?  $this->environment->getCurrentTime() : \DateTime::createFromFormat('d/m/Y', $dueDate);
            $endDate        = empty($endDate) ?  null : \DateTime::createFromFormat('d/m/Y', $endDate);
            $classification = $this->entityManager->find(Utilities::ENTITY_DIR . 'Classification', $classification);

            if($priority){
                $priority = $this->entityManager->find(Utilities::ENTITY_DIR . 'Priority', $priority);
            }

            $frontReload = false;
            if($issue->getName() != $name || $issue->getDescription() != $description ||
                $issue->getTaskStatus()->getId() != $issueStatus->getId() ||
                $issue->getClassification()->getId() != $classification->getId()){
                $frontReload = true;
            }

            if($issue->getProductVersionDev()){
                if($issue->getProductVersionDev()->getId() != $productVersionDevId){
                    $frontReload = true;
                }
            } else if($productVersionDevId){
                $frontReload = true;
            }


            if($priority && $issue->getPriority()) {
                if($priority->getId() != $issue->getPriority()->getId()){
                    $frontReload = true;
                }
            } else if($priority && !$issue->getPriority()) {
                $frontReload = true;
            } else if(!$priority && $issue->getPriority()) {
                $frontReload = true;
            }

            if($board){
                $board = $this->entityManager->find(Utilities::ENTITY_DIR . 'Project', $board);
            }else{
                $board = $issue->getProject();
            }
            if($isBoardIssue){
                $productVersion = $this->getProductVersion($productVersionId);
                $productVersionDev = $this->getProductVersion($productVersionDevId);
            } else {
                $productVersion = $this->getProductVersion($productVersionId);
                $productVersionDev = $this->getProductVersion($productVersionDevId);
            }
            $issue = InstanceProvider::getFactory()->getTask()->updateTask($issue, $board, null, $name, $this->environment->getCurrentTime(), $dueDate, self::ACTIVE, $description, $percentCompleted, null, null, $issueStatus, null, null, 'N', self::IS_ISSUE, $classification,null,	$priority, null, null, null, $productVersion, $productVersionDev, $issueResponse);

            $issue->setEndDate($endDate);
            $this->entityManager->persist($issue);
            $this->setLabelsIssue($labels, $labelsNotEditable, $issue);
            $allocationService = InstanceProvider::getService()->getAllocation();
            $allocationRepository = InstanceProvider::getRepository()->getAllocation();
            if(!empty($reporter)){
                /** @var Entities\Allocation $allocation */
                $allocation = $allocationRepository->findOneBy(array('task' => $issue->getId(), 'type' => Allocation::REPORTER));
                if(empty($allocation)){
                    $allocationService->createAllocation($issue, $reporter, Allocation::REPORTER, null, true);
                }
                else{
                    $allocationService->updateAllocation($allocation, $issue, $reporter, Allocation::REPORTER, null, true);
                }
            }
            if(!empty($sponsor)){
                $sponsorEntity	= $this->entityManager->find(Utilities::ENTITY_DIR.'Resource', $sponsor);
                $allocation = $allocationRepository->findOneBy(array('task' => $issue->getId(), 'type' => Allocation::SPONSOR));
                if(empty($allocation)){
                    $allocationService->createAllocation($issue, $sponsorEntity, Allocation::SPONSOR, null, true);
                } else {
                    $allocationService->updateAllocation($allocation, $issue, $sponsorEntity, Allocation::SPONSOR, null, true);
                }
            }

            foreach ($followers as $eachResource) {
                if(!empty($eachResource)){
                    $resourceProject = $resourceRepository->find($eachResource);
                    $allocation = $allocationRepository->findBy(array("task" => $issue->getId(), "resource" => $resourceProject->getId(),
                        'type' => Allocation::FOLLOWER));
                    if (empty($allocation)) {
                        $allocationService->createAllocation($issue, $resourceProject, Allocation::FOLLOWER, null, true);
                    }
                }
            }
            /** @var Entities\Allocation[] $allocation */
            $allocation = $allocationRepository->findBy(array("task" => $issue->getId(), 'type' => Allocation::FOLLOWER));

            foreach ($allocation as $eachAllocation) {
                if(!in_array($eachAllocation->getResource()->getId(), $followers)){
                    $this->entityManager->remove($eachAllocation);
                }
            }

            $user = $this->environment->getCurrentUserId();
            if(!empty($user)){
                $resourceEntity	= $this->entityManager->find(Utilities::ENTITY_DIR.'Resource', $user);
                $allocation = $allocationRepository->findOneBy(array('task' => $issue->getId(), 'type' => Allocation::FOLLOWER, 'resource' => $user));
                if($allocation){
                    $allocationService->updateAllocation($allocation, $issue, $resourceEntity, Allocation::FOLLOWER, null, true, $isRelevant);
                } else if($isRelevant ==  Entities\Allocation::RELEVANT) {
                    $allocationService->createAllocation($issue, $resourceEntity, Allocation::FOLLOWER, null, true, $isRelevant);
                }
            }

            $this->entityManager->flush();
            $connection->commit();


            return array('ID' => $issue->getId(), 'RELOAD' => $frontReload);
        }
        catch(Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $labels
     * @param $labelsNotEditable
     * @param $issue
     */
    protected function setLabelsIssue($labels, $labelsNotEditable, $issue)
    {
        $labels = array_unique(array_merge($labelsNotEditable, $labels));
        $this->setLabels($labels, $issue);
    }

    public function setLabels(array $labels,Entities\Task $task)
    {
        $taskLabelFactory = InstanceProvider::getFactory()->getTaskLabel();
        $labelRepository = InstanceProvider::getRepository()->getLabel();
        $taskLabelRepository = InstanceProvider::getRepository()->getTaskLabel();
        foreach ($labels as $labelId) {
            if(!empty($labelId)){
                /** @var  Entities\Label $label */
                $label = $labelRepository->find($labelId);
                $taskLabel = $taskLabelRepository	->findBy(array("task" => $task->getId(),
                    "label" => $label->getId()));
                if (empty($taskLabel)) {
                    $taskLabel = $taskLabelFactory->createTaskLabel($task, $label);
                    $this->entityManager->persist($taskLabel);
                }
            }
        }
        $this->entityManager->flush();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskLabel[] $taskLabels */
        $taskLabels = $taskLabelRepository->findBy(array("task" => $task->getId()));
        foreach ($taskLabels as $taskLabel) {
            if(!in_array($taskLabel->getLabel()->getId(), $labels)){
                $this->entityManager->remove($taskLabel);
                $this->entityManager->flush();
            }
        }
    }

    public function sendOutComment($issueId, $document, $taskTypeTargetId, $commentTypeId, $description){
        /** @var Entities\Task $issue */
        $issue = InstanceProvider::getRepository()->getTask()->find($issueId);
        /** @var Entities\TaskType $taskTypeTarget */
        $taskTypeTarget = InstanceProvider::getRepository()->getTaskType()->find($taskTypeTargetId);
        $taskStatus = $taskTypeTarget->getInitialTaskStatus();
        if(!$taskStatus){
            $taskStatus = InstanceProvider::getRepository()->getTaskStatus()->findOneBy(array('TASK_TYPE' => $taskTypeTargetId));
        }
        $issue->setTaskStatus($taskStatus);
        $issue->setCurrentTaskStatusIn(new \DateTime());
        $this->entityManager->persist($issue);
        /** @var Entities\CommentType $commentType */
        $commentType = InstanceProvider::getRepository()->getCommentType()->find($commentTypeId);
        $comment = InstanceProvider::getFactory()->getComments()->createComments($issue, $description,$commentType);
        $this->entityManager->persist($comment);
        $this->entityManager->flush();
        if($document){
            Utilities::getAttachmentService()->addDocumentComment($comment->getId(), $document);
        }
        if($taskStatus->getControl() == 'H'){
            $this->sendMailOnAvailableIssue($issue->getId(), $description);
        }else{
            Utilities::getCommentService()->sendsMailAdvisingAboutNewIssueComment($issue->getId(), $description);
        }
    }

    public function sendMailOnAvailableIssue($issueId, $description = null){
        $requestHTMLBuild = true;
        $taskRepository = InstanceProvider::getRepository()->getTask();
        $resourceId    = $this->environment->getCurrentUserId();
        $emails         = $taskRepository->getRelatedResourcesEmailsByIssueId($issueId, $resourceId);
        $issue          = $taskRepository->find($issueId);
        $issueName      = $issue->getName();
        $reporter       = DevOpsUtilities::getResourceNameForSendingMail($issueId, Entities\Allocation::TYPE_REPORTER);
        $sponsor        = DevOpsUtilities::getResourceNameForSendingMail($issueId, Entities\Allocation::TYPE_SPONSOR);
        $organizationName = InstanceProvider::getRepository()->getOrganization()->find($issue->getOrganization())->getName();
        $commenterName  = $this->environment->getCurrentUser()->getName();
        $issueTitle     = $issueId." - ".$issueName;
        $taskIds        = $taskRepository->getRelationedTasks($issueId);
        $date           = $this->environment->getCurrentTime()->format(Utilities::FORMAT_DATE_DEFAULT_NO_SECS);
        $productName    = $issue->getProject()->getName();
        $mailMessage = '';
        $additionalMessage = '(sem necessidade de implementação)';

        foreach ($taskIds as $taskId) {
            /** @var Entities\Task $task */
            $task = $taskRepository->find($taskId['ID']);
            if($task){
                $mailMessage .= InstanceProvider::getMailIssueMail()->buildBodyMessage( $task);
            }
        }
        if(!empty($emails)){
            foreach($emails as $email){
                InstanceProvider::getMailIssueMail()->sendAvailableMail($email['EMAIL'],$issueTitle , $commenterName, $date, $mailMessage,
                    '.', $reporter, $sponsor, $issueName, $issueId, $productName, $description, $organizationName, $requestHTMLBuild, null, false, false, null, $additionalMessage);
            }
        }
    }
}
